@@ -47,6 +47,7 @@
 #include "hooks.h"
 #include "fds.h"
 #include "memcount.h"
+#include "spamfilter.h"
 
 aMotd      *motd;
 aMotd      *helpfile;           /* misnomer, aMotd could be generalized */
@@ -59,6 +60,7 @@ char Network_Name[HOSTLEN+1];
 char Services_Name[HOSTLEN+1];
 char Stats_Name[HOSTLEN+1];
 char NS_Register_URL[TOPICLEN+1];
+char SpamFilter_URL[TOPICLEN+1];
 char Network_Kline_Address[HOSTLEN+1];
 char Local_Kline_Address[HOSTLEN+1];
 char Staff_Address[HOSTLEN+1];
@@ -102,6 +104,9 @@ extern void     read_help(char *);          /* defined in s_serv.c */
 extern void     init_globals();
 extern int      klinestore_init(int);    /* defined in klines.c */
 
+extern int uhm_type;
+extern int uhm_umodeh;
+
 char        **myargv;
 char        configfile[PATH_MAX] = {0};     /* Server configuration file */
 
@@ -870,6 +875,8 @@ main(int argc, char *argv[])
         }
     }
 
+    load_settings();
+
     /* init the modules, load default modules! */
     init_modules();
 
@@ -896,6 +903,10 @@ main(int argc, char *argv[])
         
     NOW = time(NULL);
         
+#ifdef SPAMFILTER
+    load_spamfilter();
+#endif
+
 #ifdef USE_SSL
     printf("Trying to initialize ssl...\n");
     if(!(ssl_capable = ssl_init()))
@@ -1311,6 +1322,8 @@ memcount_ircd(MCircd *mc)
     mc->s_confbuf.c++;
     mc->s_confbuf.m += sizeof(NS_Register_URL);
     mc->s_confbuf.c++;
+    mc->s_confbuf.m += sizeof(SpamFilter_URL);
+    mc->s_confbuf.c++;
     mc->s_confbuf.m += sizeof(Network_Kline_Address);
     mc->s_confbuf.c++;
     mc->s_confbuf.m += sizeof(Local_Kline_Address);
@@ -1326,3 +1339,77 @@ memcount_ircd(MCircd *mc)
     return 0;
 }
 
+/* load_settings - Load the persistent settings
+ *                   Returns: 1 = Success
+ *                            0 = Failure
+ */
+int load_settings()
+{
+    FILE *fp;
+    char tmp[PATH_MAX];
+    char line[1024];
+    char *para[MAXPARA + 1];
+    int parc;
+
+    ircsprintf(tmp, "%s/settings.txt", dpath);
+    if(!(fp = fopen(tmp, "r")))
+        return 0; /* Can't open file! */
+
+    while(fgets(line, sizeof(line), fp))
+    {
+        char *tmp = strchr(line, '\n');
+        if(!tmp)
+            break;
+        *tmp = '\0';
+        tmp = line;
+        parc = 0;
+        while(*tmp)
+        {
+            while(*tmp==' ')
+                *tmp++ = '\0';
+
+            if(*tmp==':')
+            {
+                para[parc++] = tmp + 1;
+                break;
+            }
+            para[parc++] = tmp;
+            while(*tmp && *tmp!=' ')
+                tmp++;
+        }
+        para[parc + 1] = NULL;
+        if(!mycmp(para[0], "uhm_type") && parc > 1)
+        {
+            uhm_type = atoi(para[1]);
+        }
+        if(!mycmp(para[0], "uhm_umodeh") && parc > 1)
+        {
+            uhm_umodeh = atoi(para[1]);
+        }
+    }
+    fclose(fp);
+
+    return 1;
+}
+
+/* save_settings - Save the persistent settings file
+ *                   Returns: 1 = Success
+ *                            0 = Failure
+ */
+int save_settings()
+{
+    char tmp[PATH_MAX];
+    FILE *fp;
+
+    ircsprintf(tmp, "%s/settings.txt", dpath);
+    fp = fopen(tmp, "w");
+    if(!fp)
+        return 0;
+
+    fprintf(fp, "uhm_type %d\n", uhm_type);
+    fprintf(fp, "uhm_umodeh %d\n", uhm_umodeh);
+
+    fclose(fp);
+
+    return 1;
+}
\ No newline at end of file
