@@ -81,12 +81,20 @@ extern Link *find_channel_link(Link *, aChannel *);
 #define RWO_MATCHES 0x2000
 #define RWO_CHANNEL 0x4000
 #define RWO_PROB    0x8000
+#define RWO_MASKED_HOST   0x10000
+#define RWO_UNMASKED_HOST   0x20000
 
 /* miscellaneous flags */
 #define RWC_SHOWIP  0x0001  /* WHO compatibility */
 #define RWC_CHANNEL 0x0002  /* WHO compatibility */
 #define RWC_TIME    0x0004  /* show timing stats */
 
+#ifdef USER_HOSTMASKING
+#define RWHO_HOST(cptr) IsUmodeH(cptr)?cptr->user->mhost:cptr->user->host
+#else
+#define RWHO_HOST(cptr) cptr->user->host
+#endif
+
 static const char *rwho_help[] = {
     "RWHO <[+|-]matchflags>[/<outputflags>[:<cookie>]] <args>",
     "Match flags are specified like channel modes,",
@@ -123,7 +131,13 @@ static const char *rwho_help[] = {
     "  :<cookie>     - supplied cookie (useful for scripts)",
     "  n             - user's nick",
     "  u             - user's username",
+#ifdef USER_HOSTMASKING
+    "  h             - user's current host",
+    "  H             - user's masked host",
+    "  R             - user's real/unmasked host",
+#else
     "  h             - user's host",
+#endif
     "  i             - user's IP",
     "  s             - user's server",
     "  f             - standard WHO flags (GH*%@+)",
@@ -188,7 +202,7 @@ static struct {
 #endif
 } rwho_opts;
 
-static char rwhobuf[1024];
+static char rwhobuf[2048];
 static char scratch[1024];
 
 
@@ -766,6 +780,10 @@ static int rwho_parseopts(aClient *sptr, int parc, char *parv[])
             case 'n': rwho_opts.rplfields |= RWO_NICK; sfl++; break;
             case 'u': rwho_opts.rplfields |= RWO_USER; sfl++; break;
             case 'h': rwho_opts.rplfields |= RWO_HOST; sfl++; break;
+#ifdef USER_HOSTMASKING
+            case 'H': rwho_opts.rplfields |= RWO_MASKED_HOST; sfl++; break;
+            case 'R': rwho_opts.rplfields |= RWO_UNMASKED_HOST; sfl++; break;
+#endif
             case 'i': rwho_opts.rplfields |= RWO_IP; sfl++; break;
             case 's': rwho_opts.rplfields |= RWO_SERVER; sfl++; break;
             case 'f': rwho_opts.rplfields |= RWO_FLAGS; sfl++; break;
@@ -971,6 +989,15 @@ static int rwho_match(aClient *cptr, int *failcode, aClient **failclient)
 	    return 0;
     }
 
+#ifdef USER_HOSTMASKING
+    if ((rwho_opts.check[0] & RWM_HOST) &&
+        (rwho_opts.host_func[0](rwho_opts.host_pat[0], cptr->user->host) && rwho_opts.host_func[0](rwho_opts.host_pat[0], cptr->user->mhost)))
+        return 0;
+
+    if ((rwho_opts.check[1] & RWM_HOST) &&
+        (!rwho_opts.host_func[1](rwho_opts.host_pat[1], cptr->user->host) || !rwho_opts.host_func[1](rwho_opts.host_pat[1], cptr->user->mhost)))
+        return 0;
+#else
     if ((rwho_opts.check[0] & RWM_HOST) &&
         rwho_opts.host_func[0](rwho_opts.host_pat[0], cptr->user->host))
         return 0;
@@ -978,6 +1005,7 @@ static int rwho_match(aClient *cptr, int *failcode, aClient **failclient)
     if ((rwho_opts.check[1] & RWM_HOST) &&
         !rwho_opts.host_func[1](rwho_opts.host_pat[1], cptr->user->host))
         return 0;
+#endif
 
 #ifdef RWHO_PROBABILITY
     if ((rwho_opts.check[0] | rwho_opts.check[1]) &
@@ -1124,7 +1152,7 @@ static void rwho_reply(aClient *cptr, aClient *ac, char *buf, chanMember *cm)
         if (rwho_opts.misc & RWC_SHOWIP)
             src = ac->hostip;
         else
-            src = ac->user->host;
+            src = RWHO_HOST(ac);
 
         ircsprintf(buf, getreply(RPL_WHOREPLY), me.name, cptr->name,
                    rwho_opts.chptr ? rwho_opts.chptr->chname : chname,
@@ -1150,12 +1178,29 @@ static void rwho_reply(aClient *cptr, aClient *ac, char *buf, chanMember *cm)
     }
 
     if (rwho_opts.rplfields & RWO_HOST)
+    {
+        src = RWHO_HOST(ac);
+        *dst++ = ' ';
+        while (*src)
+            *dst++ = *src++;
+    }
+
+#ifdef USER_HOSTMASKING
+    if (rwho_opts.rplfields & RWO_MASKED_HOST)
+    {
+        src = ac->user->mhost;
+        *dst++ = ' ';
+        while (*src)
+            *dst++ = *src++;
+    }
+    if (rwho_opts.rplfields & RWO_UNMASKED_HOST)
     {
         src = ac->user->host;
         *dst++ = ' ';
         while (*src)
             *dst++ = *src++;
     }
+#endif
 
     if (rwho_opts.rplfields & RWO_IP)
     {
