@@ -112,6 +112,8 @@ void init_globals()
     strncpyzt(Stats_Name, DEFAULT_STATS_NAME, sizeof(Stats_Name));
     strncpyzt(NS_Register_URL, DEFAULT_NS_REGISTER_URL,
               sizeof(NS_Register_URL));
+    strncpyzt(SpamFilter_URL, DEFAULT_SPAMFILTER_URL,
+              sizeof(SpamFilter_URL));
     strncpyzt(Network_Kline_Address, DEFAULT_NKLINE_ADDY,
                                             sizeof(Network_Kline_Address));
     strncpyzt(Local_Kline_Address, DEFAULT_LKLINE_ADDY,
@@ -390,13 +392,13 @@ find_oper(char *name, char *username, char *sockhost, char *hostip)
 
     for(aoper = opers; aoper; aoper = aoper->next)
     {
-        if (aoper->legal == -1)
+        if (aoper->legal == -1 || mycmp(name, aoper->nick))
             continue;
 
         for(i = 0; aoper->hosts[i]; i++)
         {
-            if(!mycmp(name, aoper->nick) && (!match(aoper->hosts[i], userhost)
-                    || !match(aoper->hosts[i], userip)))
+            if(!match(aoper->hosts[i], userhost)
+                    || !match(aoper->hosts[i], userip))
                 return aoper;
 	    if(strchr(aoper->hosts[i], '/'))
 	    {
@@ -488,7 +490,7 @@ set_effective_class(aClient *cptr)
     cptr->class->links++;
     return;
 }
-    
+
 
 /* find the first (best) I line to attach.
  * rewritten in feb04 for the overdue death of aConfItem
@@ -496,7 +498,7 @@ set_effective_class(aClient *cptr)
  * Rewritten again in Mar04 to optimize and get rid of deceptive logic.
  * Whoever wrote this originally must have been drunk...  -Quension
  */
-int 
+int
 attach_Iline(aClient *cptr, struct hostent *hp, char *sockhost)
 {
     aAllow *allow;
@@ -520,7 +522,7 @@ attach_Iline(aClient *cptr, struct hostent *hp, char *sockhost)
         add_local_domain(namehost, len - strlen(namehost));
     }
 
-    for (allow = allows; allow; allow = allow->next) 
+    for (allow = allows; allow; allow = allow->next)
     {
         if(allow->legal == -1)
             continue;
@@ -604,7 +606,7 @@ attach_Iline(aClient *cptr, struct hostent *hp, char *sockhost)
  * rewrote to remove the "ONE" lamity *BLEH* I agree with comstud on
  * this one. - Dianora
  */
-static int 
+static int
 attach_iline(aClient *cptr, aAllow *allow, char *uhost, int doid)
 {
     if(allow->class->links >= allow->class->maxlinks)
@@ -613,7 +615,7 @@ attach_iline(aClient *cptr, aAllow *allow, char *uhost, int doid)
     if (doid)
         cptr->flags |= FLAGS_DOID;
     get_sockhost(cptr, uhost);
-    
+
     cptr->user->allow = allow;
     allow->clients++;
 
@@ -627,8 +629,8 @@ attach_iline(aClient *cptr, aAllow *allow, char *uhost, int doid)
 static int oper_access[] =
 {
     ~0,            '*',
-    OFLAG_LOCAL,   'o',
     OFLAG_GLOBAL,  'O',
+    OFLAG_LOCAL,   'o',
     OFLAG_REHASH,  'r',
     OFLAG_DIE,     'D',
     OFLAG_RESTART, 'R',
@@ -1042,6 +1044,11 @@ confadd_options(cVar *vars[], int lnum)
             tmp->type = NULL;
             strncpyzt(NS_Register_URL, tmp->value, sizeof(NS_Register_URL));
         }
+        else if(tmp->type && (tmp->type->flag & OPTF_SPAMFILTERURL))
+        {
+            tmp->type = NULL;
+            strncpyzt(SpamFilter_URL, tmp->value, sizeof(SpamFilter_URL));
+        }
         else if(tmp->type && (tmp->type->flag & OPTF_MAXCHAN))
         {
             tmp->type = NULL;
@@ -1137,6 +1144,11 @@ confadd_options(cVar *vars[], int lnum)
             tmp->type = NULL;
             tswarndelta = atoi(tmp->value);
         }
+	else if(tmp->type && (tmp->type->flag & OPTF_REMREHOK))
+	{
+            tmp->type = NULL;
+            new_confopts |= FLAGS_REMREHOK;
+        }
     }
     return lnum;
 }
@@ -1802,7 +1814,7 @@ confadd_modules(cVar *vars[], int lnum)
     }
     return lnum;
 }
-    
+
 
 /* set_classes
  * after loading the config into temporary lists, we must
@@ -1833,7 +1845,7 @@ set_classes(void)
      * You may be wondering why we're doing this here and appearently
      * again in our merge routines!  well, this is for sanity.  if
      * for whatever reason we dont have a class for each definition here,
-     * back out of the conf load immediately and we wont have distroyed 
+     * back out of the conf load immediately and we wont have distroyed
      * or overwritten any of our active data.
      * After we run our merge_classes() routine at the start of our
      * merge, then some of these classes will update currently active
@@ -2050,7 +2062,7 @@ merge_allows()
     }
     return;     /* this one is easy */
 }
-    
+
 static void
 merge_opers()
 {
@@ -2123,7 +2135,7 @@ static void
 merge_ports()
 {
     aPort *aport, *old_port, *ptrn;
-    
+
     if(forked)
         close_listeners();      /* marks ports for deletion */
 
@@ -2322,7 +2334,7 @@ clear_newconfs()
 
 /*
  * rehash
- * 
+ *
  * Actual REHASH service routine. Called with sig == 0 if it has been
  * called as a result of an operator issuing this command, else assume
  * it has been called as a result of the server receiving a HUP signal.
@@ -2333,8 +2345,13 @@ int rehash(aClient *cptr, aClient *sptr, int sig)
     int         i;
     char       *conferr;
 
-    if (sig == SIGHUP) 
+    if (sig == SIGHUP)
     {
+#ifdef USE_SSL
+		/* Rehash SSL so we can automate certificate renewals and updates externally, i.e. from a cron job --xPsycho */
+		sendto_ops("Got signal SIGHUP, rehashing SSL");
+		ssl_rehash();
+#endif
         sendto_ops("Got signal SIGHUP, reloading ircd conf. file");
         remove_userbans_match_flags(UBAN_NETWORK, 0);
         /* remove all but kill {} blocks from conf */
@@ -2346,7 +2363,7 @@ int rehash(aClient *cptr, aClient *sptr, int sig)
     }
 
     for (i = 0; i <= highest_fd; i++)
-        if ((acptr = local[i]) && !IsMe(acptr)) 
+        if ((acptr = local[i]) && !IsMe(acptr))
         {
             /*
              * Nullify any references from client structures to this host
@@ -2371,7 +2388,11 @@ int rehash(aClient *cptr, aClient *sptr, int sig)
 
     if(initconf(configfile) == -1)
     {
-        sendto_realops("Rehash Aborted");
+        if (sptr->name == me.name)
+            sendto_realops("Rehash Aborted");
+        else
+            sendto_one(sptr, ":%s NOTICE %s :Rehash Aborted", me.name, sptr->name);
+
         clear_newconfs();
         return 1;
     }
@@ -2379,11 +2400,15 @@ int rehash(aClient *cptr, aClient *sptr, int sig)
     conferr = finishconf();
     if (conferr)
     {
-        sendto_realops("Rehash Aborted: %s", conferr);
+        if (sptr->name == me.name)
+            sendto_realops("Rehash Aborted: %s", conferr);
+        else
+            sendto_one(sptr, ":%s NOTICE %s :Rehash Aborted: %s", me.name, sptr->name, conferr);
+
         clear_newconfs();
         return 1;
     }
-    
+
     merge_confs();
     build_rplcache();
     nextconnect = 1;    /* reset autoconnects */
@@ -2463,6 +2488,113 @@ static int lookup_confhost(aConnect *aconn)
     return -1;
 }
 
+/* oflagtotext()
+ * Return the oflags in human readable format.
+ * Oct06 -Kobi_S
+ */
+char *oflagtotext(int oflags)
+{
+    static char res[BUFSIZE + 1];
+    int *i, flag, len = 0;
+
+    for (i=oper_access; (flag = *i); i+=2)
+        if ((oflags & flag) == flag)
+        {
+            res[len++] = (char)(*(i+1));
+            oflags &= ~flag;
+        }
+
+    res[len++] = 0;
+
+    return res;
+}
+
+/* cflagtotext()
+ * Return the cflags in human readable format.
+ * Sep08 -Kobi_S
+ */
+char *cflagtotext(int cflags, int uflags)
+{
+    static char res[BUFSIZE + 1];
+    int *i, flag, len = 0;
+
+    for (i=server_info; (flag = *i); i+=2)
+        if ((cflags & flag) == flag)
+        {
+            res[len++] = (char)(*(i+1));
+            cflags &= ~flag;
+        }
+
+    if(!len)
+        res[len++] = '-';
+
+    if(uflags)
+    {
+        res[len++] = '/';
+
+        for (i=server_uflags; (flag = *i); i+=2)
+            if ((uflags & flag) == flag)
+            {
+                res[len++] = (char)(*(i+1));
+                uflags &= ~flag;
+            }
+    }
+
+    res[len++] = 0;
+
+    return res;
+}
+
+/* iflagtotext()
+ * Return the iflags in human readable format.
+ * Sep08 -Kobi_S
+ */
+char *iflagtotext(int iflags)
+{
+    static char res[BUFSIZE + 1];
+    int len = 0;
+
+    if(iflags & CONF_FLAGS_I_OPERPORT)
+        res[len++] = 'm';
+    if(iflags & CONF_FLAGS_NOTHROTTLE)
+        res[len++] = 'T';
+    if(iflags & CONF_FLAGS_FORCEFLOOD)
+        res[len++] = 'F';
+    if(iflags & CONF_FLAGS_SKIPCLONES)
+        res[len++] = 'C';
+
+    if(!len)
+        res[len++] = '-';
+
+    res[len++] = 0;
+
+    return res;
+}
+
+/* pflagtotext()
+ * Return the pflags in human readable format.
+ * May 20 - rasengan
+ */
+char *pflagtotext(int pflags)
+{
+    static char res[BUFSIZE + 1];
+    int len = 0;
+
+    if(pflags & CONF_FLAGS_P_SSL)
+        res[len++] = 'S';
+    if(pflags & CONF_FLAGS_P_NODNS)
+        res[len++] = 'n';
+    if(pflags & CONF_FLAGS_P_NOIDENT)
+        res[len++] = 'i';
+
+    if(!len)
+        res[len++] = '-';
+
+    res[len++] = 0;
+
+    return res;
+}
+
 u_long
 memcount_s_conf(MCs_conf *mc)
 {
@@ -2595,4 +2727,3 @@ memcount_s_conf(MCs_conf *mc)
 
     return mc->total.m;
 }
-
